﻿using Property.Model.Entity;
using Property.Model.Extend;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Property.Repository
{
    /// <summary>
    /// 耗材入库单明细仓储类
    /// </summary>
    public partial class RB_Supplies_StockInDetailRepository : RepositoryBase<RB_Supplies_StockInDetail>
    {
        /// <summary>
        /// 获取入库明细数据
        /// </summary>
        /// <param name="dmodel"></param>
        /// <returns></returns>
        public List<RB_Supplies_StockInDetail_Extend> GetList(RB_Supplies_StockInDetail_Extend dmodel)
        {
            string where = " where 1=1 ";
            where += string.Format(" AND ssd.{0}={1}", nameof(RB_Supplies_StockInDetail_Extend.Status), 0);
            where += $@" and ssd.RB_Group_Id={dmodel.RB_Group_Id}";            

            if (dmodel.StockInId > 0) {
                where += " and ssd." + nameof(RB_Supplies_StockInDetail_Extend.StockInId) + "=" + dmodel.StockInId;
            }
            if (dmodel.SuppliesId > 0) {
                where += " and ssd." + nameof(RB_Supplies_StockInDetail_Extend.SuppliesId) + "=" + dmodel.SuppliesId;
            }
            if (!string.IsNullOrEmpty(dmodel.StockInIdStr)) {
                where += " and ssd." + nameof(RB_Supplies_StockInDetail_Extend.StockInId) + " in(" + dmodel.StockInIdStr + ")";
            }
            
            string sql = $@" select ssd.*,sm.Name as SuppliesName,sm.SuppliesNum,sm.BrandName,sm.SpecificationName,sm.GoodsCategoryName,sm.SuppliesModel,sm.Units,pc.Name as CategoryName
from rb_supplies_stockindetail ssd 
left join rb_supplies_material sm on ssd.SuppliesId=sm.Id
left join rb_property_category pc on pc.Id=sm.CategoryId
{where} order by ssd.Id desc";
            return Get<RB_Supplies_StockInDetail_Extend>(sql).ToList();
        }

        /// <summary>
        /// 获取近期过期的物料列表+数量
        /// </summary>
        /// <param name="rB_Group_Id"></param>
        /// <param name="suppliesIds"></param>
        /// <returns></returns>
        public List<RB_Supplies_StockInDetail_Extend> GetRecentExpiresSuppliesNumList(int rB_Group_Id, string suppliesIds, string expirationDate)
        {
            string sql = $@"SELECT sd.SuppliesId,COUNT(0) as Number FROM rb_supplies_stockinvalidity v
INNER JOIN rb_supplies_stockindetail sd on v.StockinDetailId = sd.Id
INNER JOIN rb_supplies_stockin s on sd.StockInId = s.Id
WHERE v.`Status`=0 and v.RB_Group_Id ={rB_Group_Id} AND s.`Status`=0 and s.StockInState =1 and sd.SuppliesId in({suppliesIds}) and v.IsOut =2 and v.ExpirationDate <='{expirationDate} 23:59:59'
GROUP BY sd.SuppliesId";
            return Get<RB_Supplies_StockInDetail_Extend>(sql).ToList();
        }

        /// <summary>
        /// 获取耗材有效期分页列表
        /// </summary>
        /// <param name="pageIndex"></param>
        /// <param name="pageSize"></param>
        /// <param name="count"></param>
        /// <param name="demodel"></param>
        /// <returns></returns>
        public List<RB_Supplies_StockInDetail_Extend> GetSuppliesValidityPageList(int pageIndex, int pageSize, out long count, RB_Supplies_StockIn_Extend demodel)
        {
            string where = "";
            if (demodel.RB_Group_Id > 0) {
                where += $" and v.RB_Group_Id ={demodel.RB_Group_Id}";
            }
            if (demodel.WarehouseId > 0) {
                where += $" and s.WarehouseId ={demodel.WarehouseId}";
            }
            if (!string.IsNullOrEmpty(demodel.StockInNum)) {
                where += $" and s.StockInNum like '%{demodel.StockInNum}%'";
            }
            if (!string.IsNullOrEmpty(demodel.SuppliesName)) {
                where += $" and w.`Name` like '%{demodel.SuppliesName}%'";
            }
            if (!string.IsNullOrEmpty(demodel.SuppliesNum)) {
                where += $" and m.SuppliesNum like '%{demodel.SuppliesNum}%'";
            }
            if (demodel.SuppliesId > 0) {
                where += $" and sd.SuppliesId ={demodel.SuppliesId}";
            }
            if (!string.IsNullOrEmpty(demodel.StartTime)) {
                where += $" and v.ExpirationDate >='{demodel.StartTime}'";
            }
            if (!string.IsNullOrEmpty(demodel.EndTime)){
                where += $" and v.ExpirationDate <='{demodel.EndTime}'";
            }

            string sql = $@"SELECT v.Id,DATE_FORMAT(v.ExpirationDate,'%Y-%m-%d') as ExpirationDate,sd.StockInId,s.StockInNum,DATE_FORMAT(s.StockInDate,'%Y-%m-%d') as StockInDate,w.`Name` as WarehouseName,sd.SuppliesId,m.SuppliesNum,m.`Name` as SuppliesName,m.SpecificationName,sd.UnitPrice FROM rb_supplies_stockinvalidity v
INNER JOIN rb_supplies_stockindetail sd on v.StockinDetailId = sd.Id
INNER JOIN rb_supplies_stockin s on sd.StockInId = s.Id
INNER JOIN rb_supplies_material m on sd.SuppliesId = m.Id
LEFT JOIN rb_supplies_warehouse w on s.WarehouseId = w.Id
WHERE v.`Status`=0 AND s.`Status`=0 and s.StockInState =1 and v.IsOut =2 {where}
";
            return GetPage<RB_Supplies_StockInDetail_Extend>(pageIndex, pageSize, out count, sql).ToList();
        }
    }
}
